// Author
// http://java-swing-tips.blogspot.cl/2008/10/using-rubber-band-selection-in-jlist.html

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import java.util.List;
import javax.swing.*;
import javax.swing.border.*;

public class SelectableGrid extends JPanel{
    
    private int cols;
    private int rows;

    /**
     *
     * @param pixHeight
     * @param driver_class
     */
    public SelectableGrid(int pixHeight, Tileset driver_class){
        super(new BorderLayout());
        this.setOpaque(false);
        rows = driver_class.getTilesetDimension().height;
        cols = driver_class.getTilesetDimension().width;
        
        DefaultListModel<ListItem> model = new DefaultListModel<>();
        for(int i=0; i<cols*rows; i++){
            model.addElement(new ListItem());
        }
        add(new RubberBandSelectionList<ListItem>(model,rows,driver_class));
        this.setBackground(Color.red);
    }

}

class ListItem {
    public final ImageIcon nicon = new ImageIcon(getClass().getResource("./image/Transparent.png"));
    public final ImageIcon sicon = new ImageIcon(getClass().getResource("./image/Selected.png"));
    protected ListItem() {}
}

class RubberBandSelectionList<E extends ListItem> extends JList<E> {
    private static final AlphaComposite ALPHA = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, .1f);
    private transient RubberBandingListener rbl;
    private Color rubberBandColor;
    private int rows;
    private final Path2D rubberBand = new Path2D.Double();
    private Tileset DrivingTilesetClass;
    protected RubberBandSelectionList(ListModel<E> model, int rows, Tileset driver_class) {
        super(model);
        DrivingTilesetClass = driver_class;
        this.rows = rows;
        setVisibleRowCount(rows);
        setOpaque(false);
    }
    @Override public void updateUI() {
        setSelectionForeground(null); //Nimbus
        setSelectionBackground(null); //Nimbus
        setCellRenderer(null);
        removeMouseListener(rbl);
        removeMouseMotionListener(rbl);
        super.updateUI();
        setOpaque(false);

        rubberBandColor = makeRubberBandColor(getSelectionBackground());
        setLayoutOrientation(JList.HORIZONTAL_WRAP);
        setVisibleRowCount(rows);
        setFixedCellWidth(32);
        setFixedCellHeight(32);
        //setBorder(BorderFactory.createEmptyBorder(0,0,0,0));

        setCellRenderer(new ListItemListCellRenderer<E>());
        rbl = new RubberBandingListener();
        addMouseMotionListener(rbl);
        addMouseListener(rbl);
    }
    @Override protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g.create();
        g2.setPaint(getSelectionBackground());
        g2.draw(rubberBand);
        g2.setComposite(ALPHA);
        g2.setPaint(rubberBandColor);
        g2.fill(rubberBand);
        g2.dispose();
    }
    private static Color makeRubberBandColor(Color c) {
        int r = c.getRed();
        int g = c.getGreen();
        int b = c.getBlue();
        return r > g ? r > b ? new Color(r, 0, 0) : new Color(0, 0, b)
                     : g > b ? new Color(0, g, 0) : new Color(0, 0, b);
    }
    private class RubberBandingListener extends MouseAdapter {
        private final Point srcPoint = new Point();
        @Override public void mouseDragged(MouseEvent e) {
            JList l = (JList) e.getComponent();
            l.setFocusable(true);
            Point destPoint = e.getPoint();
            rubberBand.reset();
            rubberBand.moveTo(srcPoint.x,  srcPoint.y);
            rubberBand.lineTo(destPoint.x, srcPoint.y);
            rubberBand.lineTo(destPoint.x, destPoint.y);
            rubberBand.lineTo(srcPoint.x,  destPoint.y);
            rubberBand.closePath();
            l.setSelectedIndices(getIntersectsIcons(l, rubberBand));
            //JDK 1.8.0: l.setSelectedIndices(IntStream.range(0, l.getModel().getSize()).filter(i -> rubberBand.intersects(l.getCellBounds(i, i))).toArray());
            l.repaint();
        }
        @Override public void mouseReleased(MouseEvent e) {
            rubberBand.reset();
            Component c = e.getComponent();
            c.setFocusable(true);
            c.repaint();
            JList l = (JList) e.getComponent();
            int[] selected_indices = l.getSelectedIndices();
            int begining_idx = selected_indices[0];
            int end_idx = selected_indices[selected_indices.length-1];
            int cols = DrivingTilesetClass.getTilesetDimension().width;
            int x_i = begining_idx%cols;
            int x_f = end_idx%cols;
            int y_i = begining_idx/cols;
            int y_f = end_idx/cols;
            DrivingTilesetClass.setSelectedTiles(x_i,x_f,y_i,y_f);
        }
        @Override public void mousePressed(MouseEvent e) {
            JList l = (JList) e.getComponent();
            int index = l.locationToIndex(e.getPoint());
            Rectangle rect = l.getCellBounds(index, index);
            if (rect.contains(e.getPoint())) {
                l.setFocusable(true);
            } else {
                l.clearSelection();
                l.getSelectionModel().setAnchorSelectionIndex(-1);
                l.getSelectionModel().setLeadSelectionIndex(-1);
                l.setFocusable(false);
            }
            srcPoint.setLocation(e.getPoint());
            l.repaint();
        }
    }
    //JDK 1.7.0
    private static int[] getIntersectsIcons(JList<?> l, Shape rect) {
        ListModel model = l.getModel();
        List<Integer> ll = new ArrayList<>(model.getSize());
        for (int i = 0; i < model.getSize(); i++) {
            if (rect.intersects(l.getCellBounds(i, i))) {
                ll.add(i);
            }
        }
        //JDK 1.8.0: return ll.stream().mapToInt(i -> i).toArray();
        int[] il = new int[ll.size()];
        for (int i = 0; i < ll.size(); i++) {
            il[i] = ll.get(i);
        }
        return il;
    }
}

class ListItemListCellRenderer<E extends ListItem> implements ListCellRenderer<E> {
    private final JPanel p = new JPanel(new BorderLayout());
    private final JLabel icon  = new JLabel((Icon) null, SwingConstants.CENTER);

    protected ListItemListCellRenderer() {
        icon.setOpaque(false);
        p.setOpaque(false);
        p.setBorder(BorderFactory.createEmptyBorder());
        p.add(icon);
    }
    @Override public Component getListCellRendererComponent(JList<? extends E> list, E value, int index, boolean isSelected, boolean cellHasFocus) {
        if (isSelected) {
            icon.setIcon(value.sicon);
        } else {
            icon.setIcon(value.nicon);
        }
        return p;
    }
}
